<?php

/**
 * Klasa do obsługi API Domeny.tv przez SOAP
 * Zawiera część dostępnych funkcji. Pozostałe można w podobny sposób oprogramować.
 * @copyright (c) 2012-2022 MSERWIS.pl
 * @author Jacek Partyka
 * @package
 */

ini_set('display_errors', 'on');
error_reporting(E_ALL);

class DTVsoapClient
{

    private $login = '';
    private $password = '';
    private $wsdl_url = 'https://www.domeny.tv/regapi/test/soap.wsdl.xml';
    public $client = null;
    private static $me;

    private function __construct()
    {
        $this->client = new SoapClient($this->wsdl_url);
    }

    /**
     * Singleton
     */
    public static function instance()
    {
        if (!self::$me) {
            self::$me = new DTVsoapClient();
        }

        return self::$me;
    }

    /**
     * Sprawdza dostępność domeny
     * @param string $domain - nazwa domeny wraz z rozszerzeniem
     */
    public function checkDomain($domain)
    {
        $result = $this->client->checkDomain(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Metoda tworzy hosta w NASK
     * @param string $host
     * @param string $ip
     * @return boolean | int
     */
    public function hostCreate($host, $ip = null)
    {
        $request = array(
            'login' => $this->login,
            'password' => $this->password,
            'host' => $host
        );

        if ($ip) {
            $request['ip'] = $ip;
        }

        $result = $this->client->hostCreate($request);
        $this->handleCommonResponses($result);
        if ($result['result'] == '1000') {
            return true;
        } else {
            return $result['result'];
        }
    }

    /**
     * Podaje wysokość środków przedpłaconych na koncie
     * @return boolean|float
     */
    public function accountBalance()
    {
        $result = $this->client->accountBalance(
            array(
                'login' => $this->login,
                'password' => $this->password
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['balance'];
        } else {
            return false;
        }
    }

    /**
     * Lista ostatnich operacji wykonanych w serwisie domeny.tv
     * @return boolean|array
     */
    public function lastOperations()
    {
        $result = $this->client->lastOperations(
            array(
                'login' => $this->login,
                'password' => $this->password
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['operations'];
        } else {
            return false;
        }
    }

    /**
     * Transferuje domenę do serwisu domeny.tv
     * DLA NIEKTÓRYCH DOMEN OPERACJA PŁATNA
     * @param string $domain
     * @param string $authinfo
     * @param int $resend
     * @param array $contact
     * @return boolean
     */
    public function transferDomain($domain, $authinfo, $resend = 0, $contact = null)
    {
        $result = $this->client->transferDomain(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'authinfo' => $authinfo,
                'resend' => $resend,
                'contact' => $contact
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Zwraca informacje o domenie .pl
     * @param string $domain
     * @param string $authinfo
     * @return array|int
     */
    public function domainInfo($domain, $authinfo = null)
    {
        $result = $this->client->domainInfo(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'authinfo' => $authinfo
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return array(
                'crDate' => $result['crDate'],
                'exDate' => $result['exDate'],
                'status' => $result['status'],
                'registrant' => $result['registrant'],
                'technical' => $result['technical'],
                'ns' => $result['ns']
            );
        } else {
            return $result['result'];
        }
    }

    /**
     * Zwraca informacje o hoście - tylko dla domen .pl
     * @param string $host
     * @return array|int
     */
    public function hostInfo($host)
    {
        $result = $this->client->hostInfo(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'host' => $host
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return array(
                'crDate' => $result['crDate'],
                'status' => $result['status'],
                'ip' => $result['ip']
            );
        } else {
            return $result['result'];
        }
    }

    /**
     * Uaktualnia informacje o IP hosta - tylko dla domen .pl
     * @param string $host
     * @return true|int
     */
    public function hostUpdate($host, $ip)
    {
        $result = $this->client->hostUpdate(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'host' => $host,
                'ip' => $ip
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return true;
        } else {
            return $result['result'];
        }
    }

    /**
     * Rejestruje domenę
     * @param string $domain
     * @param array $registrant
     * @param int $period
     * @param array $dns [NameServer1 => '', NameServer2 => '', ... , NameServer9 => '']
     * @return int
     */
    public function registerDomain($domain, $registrant, $period, $dns)
    {
        $result = $this->client->registerDomain(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'registrant' => $registrant,
                'period' => $period,
                'dns' => $dns
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Zmienia delegację domeny
     * @param string $domain
     * @param array $dns
     * @return int
     */
    public function changeDomainNameservers($domain, $dns)
    {
        $result = $this->client->changeDomainNameservers(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'dns' => $dns
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Cesja domeny
     * DLA NIEKTÓRYCH DOMEN OPERACJA PŁATNA
     * @param string $domain
     * @param array $registrant
     * @return int
     */
    public function domainCession($domain, $registrant)
    {
        $result = $this->client->domainCession(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'registrant' => $registrant
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Przedłuża domenę
     * OPERACJA PŁATNA
     * @param string $domain
     * @param int $period
     * @return type
     */
    public function domainRenew($domain, $period)
    {
        $result = $this->client->domainRenew(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'period' => $period
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Pobiera authinfo dla domen .pl
     * @param string $domain
     * @return string|int
     */
    public function domainGetAuthinfo($domain)
    {
        $result = $this->client->domainGetAuthinfo(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['authinfo'];
        } else {
            return $result['result'];
        }
    }

    /**
     * Sprawdza czy opcja na domenę istnieje
     * @param type $domain
     * @return int
     */
    public function futureCheck($domain)
    {
        $result = $this->client->futureCheck(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );

        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Zwraca informację o opcji na domenę .pl
     * @param string $domain
     * @return array|int
     */
    public function futureInfo($domain)
    {
        $result = $this->client->futureInfo(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );

        $this->handleCommonResponses($result);
        if ($result['result'] == '4002') {
            return array(
                'crDate' => $result['crDate'],
                'exDate' => $result['exDate'],
                'registrant' => $result['registrant']
            );
        } else {
            return $result['result'];
        }
    }

    /**
     * Zakłada opcję na domenę .pl
     * OPERACJA PŁATNA
     * @param string $domain
     * @param array $registrant
     * @return int
     */
    public function futureCreate($domain, $registrant)
    {
        $result = $this->client->futureCreate(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'registrant' => $registrant
            )
        );
        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Sprawdza czy domena ma aktywny serwer przekierowań.
     * UWAGA! Metoda nie sprawdza, czy domena jest wydelegowana na serwery
     * dns1.domeny.tv
     * dns2.domeny.tv
     * @param string $domain
     * @return string|int - result może być 'parking' lub 'dns', w zależności
     * czy domena jest zaparkowana na parkingu czy też można modyfikować
     * jej strefę na serwerze DNS
     */
    public function checkRedirectionServer($domain)
    {
        $result = $this->client->checkRedirectionServer(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['type'];
        } else {
            return $result['result'];
        }
    }

    /**
     * Aktywuje serwer przekierowań
     * UWAGA! Metoda nie zmienia delegacji domeny!
     * @param string $domain
     * @return int
     */
    public function activateRedirectionServer($domain)
    {
        $result = $this->client->activateRedirectionServer(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Włącza parking dla domeny
     * UWAGA! Metoda nie zmienia delegacji domeny!
     * @param string $domain
     * @return int
     */
    public function activateParking($domain)
    {
        $result = $this->client->activateParking(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Usuwa domenę zarówno z parkingu jak i z serwera przekierowań
     * UWAGA! Metoda nie zmienia delegacji domeny!
     * @param string $domain
     * @return int
     */
    public function deleteRedirectionServer($domain)
    {
        $result = $this->client->deleteRedirectionServer(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        return $result['result'];
    }

    /**
     * Ustawia rekordy DNS dla domeny z aktywnym serwerem przekierowań.
     * @param string $domain
     * @param array $records (type, subdomain, value, priority)
     * @return int
     */
    public function setDomainDNSRecords($domain, $records)
    {
        $result = $this->client->setDomainDNSRecords(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain,
                'records' => $records
            )
        );
        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['result'];
        } elseif (isset($result['message'])) {
            return array('result' => $result['result'], 'message' => $result['message']);
        } else {
            return array('result' => $result['result'], 'type' => $result['type'], 'subdomain' => $result['subdomain']);
        }
    }

    /**
     * Pobiera rekordy DNS ustawione w serwerze przekierowań
     * @param string $domain
     */
    public function getDomainDNSRecords($domain)
    {
        $result = $this->client->getDomainDNSRecords(
            array(
                'login' => $this->login,
                'password' => $this->password,
                'domain' => $domain
            )
        );
        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['records'];
        } else {
            return $result['result'];
        }
    }

    /**
     * Zwraca dostępne w serwisie domeny wraz z cenami ich rejestracji/przedłużenia/cesji/transferu
     */
    public function pricelist()
    {
        $result = $this->client->pricelist(
            array(
                'login' => $this->login,
                'password' => $this->password
            )
        );
        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['domains'];
        } else {
            return $result['result'];
        }
    }

    /**
     * Zwraca listę domen na koncie użytkownika
     */
    public function getAllDomains()
    {
        $result = $this->client->getAllDomains(
            array(
                'login' => $this->login,
                'password' => $this->password
            )
        );
        $this->handleCommonResponses($result);
        if ($result['result'] == 1000) {
            return $result['domains'];
        } else {
            return $result['result'];
        }
    }


    /**
     * Metoda rozpoznaje niektóre z typowych odpowiedzi API
     * @param array $result
     * @throws domainException
     */
    private function handleCommonResponses($result)
    {
        if (isset($result['result'])) {
            switch ((int)$result['result']) {
                case 0:
                    throw new DTVdomainException("Błąd połączenia z serwerem", 0);
                    break;

                case 16:
                    throw new DTVdomainException("Niepoprawne dane wejściowe", 16);
                    break;

                case 27:
                    throw new DTVdomainException("Niepoprawny login/hasło", 27);
                    break;

                case 23:
                    throw new DTVdomainException("Nie masz uprawnień do tej domeny", 23);
                    break;

                case 1:
                    throw new DTVdomainException("Niepoprawna nazwa domeny", 1);
                    break;
            }
        } else {
            throw new DTVdomainException("Błąd połączenia", 0);
        }
    }
}

class DTVdomainException extends Exception
{

    /**
     * Konstruktor wyjątku. Można tu dodać inne rzeczy takie jak logowanie błędów itp
     * @param string $message
     * @param int $code
     */
    public function __construct($message, $code = 0)
    {
        parent::__construct($message, $code);
    }
}












// przykłady wykorzystania:

// check:
try {

    $client = DTVsoapClient::instance();

    //sprawdzamy domeny
    /*
   try {
      $result = $client->checkDomain('domenka.pl');
      if ($result == 1000)
         echo 'Domena jest wolna do rejestracji';
      else
         echo 'Domena jest zajęta';
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // tworzenia hosta
    /*
   try {
      $result = $client->hostCreate('dns2.domeny.tv');
      if ($result === true)
         echo 'Host zarejestrowany';
      else
         echo 'Nie udało się zarejestrować hosta. Błąd nr: '.$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // sprawdzanie salda
    /*
   try {
      $result = $client->accountBalance();
      echo 'Saldo konta: '.$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // lista ostatnich operacji
    /*
   try {
      $result = $client->lastOperations();
      echo 'Ostatnie operacje: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // transfer domeny
    /*
   try {
      $result = $client->transferDomain('domenka.pl', 'abcdefg');
      if ($result == 1000 || $result == 1001)
         echo "Transfer zakończony";
      else
         echo "Wystąpił błąd: ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // informacje o domenie
    /*
   try {
      $result = $client->domainInfo('domenka.pl');
      echo 'Informacje o domenie: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // informacje o hoście
    /*
   try {
      $result = $client->hostInfo('dns1.mserwis.pl');
      echo 'Informacje o hoście: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // update ip hosta
    /*
   try {
      $result = $client->hostUpdate('dns9.mserwis.pl', '70.84.0.56');
      echo $result === true ? "Utworzono hosta" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // rejestracja domeny
    /*
   try {
      $result = $client->registerDomain('domenka.pl', array(
         'companyName' => 'MSERWIS',
         'firstName' => 'Jan',
         'lastName' => 'Kowalski',
         'address' => 'Kazimierza Wielkiego 27',
         'zip' => '50-077',
         'district' => 'dolnośląskie',
         'city' => 'Wrocław',
         'country' => 'PL',
         'phone' => '+48.100200300',
         'email' => 'jankowalski@mserwis.pl'
      ), 1, array(
         'NameServer1' => 'dns1.domeny.tv',
         'NameServer2' => 'dns2.domeny.tv'
      ));
      echo $result == 1000 ? "Utworzono domenę" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // zmiana delegacji
    /*
   try {
      $result = $client->changeDomainNameservers('domenka.pl', array(
         'NameServer1' => 'dns1.mserwis.pl',
         'NameServer2' => 'dns2.mserwis.pl'
      ));
      echo $result == 1000 ? "Zmieniono delegację" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // robimy check czy domena ma zmienione dnsy
    /*
   try {
      $result = $client->domainInfo('domenka.pl');
      echo 'Informacje o domenie: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // cesja domeny
    /*
   try {
      $result = $client->domainCession('domenka.pl', array(
         'companyName' => '',
         'firstName' => 'Jan',
         'lastName' => 'Kowalski',
         'address' => 'Kowalska 1/5',
         'zip' => '00-950',
         'district' => 'mazowieckie',
         'city' => 'Warszawa',
         'country' => 'PL',
         'phone' => '+48.100200300',
         'email' => 'kowalski@mserwis.pl'
      ));
      echo $result == 1000 ? "Cesja wykonana" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // przedłużenie domeny
    /*
   try {
      $result = $client->domainRenew('domenka.pl', 2);
      echo $result == 1000 ? "Domena przedłużona" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // sprawdzamy czy przedłużono
    /*
   try {
      $result = $client->domainInfo('domenka.pl');
      echo 'Informacje o domenie: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // pobieramy authinfo
    /*
   try {
      $result = $client->domainGetAuthinfo('domenka.pl');
      echo 'Authinfo: '.print_r($result, true);
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // sprawdzamy dostępność opcji
    /*
   try {

      $result = $client->futureCheck('domenka.pl');
      if ($result == 4002)
         echo "Opcja założona";
      elseif ($result == 4001)
         echo "Opcja nie istnieje";
      else
         echo "Wystąpił błąd nr ".$result;

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // zakładamy opcję
    /*
   try {
      $result = $client->futureCreate('domenka.pl', array(
         'companyName' => '',
         'firstName' => 'Jan',
         'lastName' => 'Kowalski',
         'address' => 'Kowalska 1/5',
         'zip' => '00-950',
         'district' => 'mazowieckie',
         'city' => 'Warszawa',
         'country' => 'PL',
         'phone' => '+48.100200300',
         'email' => 'kowalski@mserwis.pl'
      ));
      echo $result == 1000 ? "Opcja założona" : "Wystąpił błąd nr ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // informacja o futurze
    /*
   try {

      $result = $client->futureInfo('domenka.pl');
      print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */

    // sprawdzenie serwera przekierowań
    /*
   try {

      $result = $client->checkRedirectionServer('domenka.pl');
      print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // aktywowanie serwera przekierowań
    /*
   try {

      $result = $client->activateRedirectionServer('domenka.pl');
      print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // sprawdzenie serwera przekierowań
    /*
   try {

      $result = $client->checkRedirectionServer('domenka.pl');
      print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // aktywowanie parkingu
    /*
   try {

      $result = $client->activateParking('domenka.pl');
      print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // pobieranie rekordów DNS
    /*
   try {

      $result = $client->getDomainDNSRecords('domenka.pl');
      if (is_numeric($result))
         echo "Wystąpił błąd: ".$result;
      else
         print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // ustawianie rekordów DNS
    /*
   try {

      $result = $client->setDomainDNSRecords('domenka.pl', array(
         array('type' => 'A', 'value' => '8.8.8.8', 'subdomain' => '@'),
         array('type' => 'A', 'value' => '8.8.8.8', 'subdomain' => 'www'),
         array('type' => 'CNAME', 'value' => 'www', 'subdomain' => 'mail')
      ));
      if ($result == 1000)
         echo "Rekordy ustawione";
      else
         echo "Wystąpił błąd: ".$result;
   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }
   */

    // sprawdzamy ponownie
    /*
   try {

      $result = $client->getDomainDNSRecords('domenka.pl');
      if (is_numeric($result))
         echo "Wystąpił błąd: ".$result;
      else
         print_r($result);

   }
   catch (DTVdomainException $e) {
      echo $e->getMessage();
   }

   */
} catch (Exception $ee) {
    $ee->getMessage();
    print_r($client->client);
}
